$AppName = Get-VstsInput -Name "AppName" -Default ""; 
$Action = Get-VstsInput -Name "Action" -Default "";
$DBName = Get-VstsInput -Name "DBName" -Default "";
$MSIName = Get-VstsInput -Name "MSIName" -Default "";
$FolderProductcode = Get-VstsInput -Name "FolderProductcode" -Default "";
$MSIArguments = Get-VstsInput -Name "MSIArguments" -Default " ";
$Timeout = Get-VstsInput -Name "Timeout" -Default 300;

Write-Output "Action selected - $Action"		
Write-Output "Application name - $AppName"	
Write-Output "Database name - $DBName"
Write-Output "MSI name - $MSIName"
Write-Output "Folder or Productcode - $FolderProductcode"
Write-Output "MSI arguments - $MSIArguments"
Write-Output "Timeout in sec - $Timeout"

Write-Output "Start Deploy"
    
$DataStamp = get-date -Format yyyyMMddTHHmmss
$LogPath = $Env:SYSTEM_DEFAULTWORKINGDIRECTORY


$MSIArguments = $MSIArguments -replace "`t|`n|`r"," "

$timeouted = $null 
	
if ($Action -eq "uninstall")
{
	Write-Output "Uninstall App - ($AppName)......."   

	$uninstall = 0

	# Check productcode and MSI exists
	if ($FolderProductcode -like "{*}"){	
		
		Write-Output "Search guid: " + $FolderProductcode 
		$properties = "identifyingnumber","name","vendor","version"
		Get-ChildItem HKLM:SOFTWARE\Microsoft\Windows\CurrentVersion\Installer\UserData\*\Products | ForEach-Object {
			$root  = $_.PsPath
			$_.GetSubKeyNames() | ForEach-Object {
				try {
					$RegKeyPath = (Join-Path -Path (Join-Path -Path $root -ChildPath $_) -ChildPath InstallProperties)
					$obj = Get-ItemProperty -Path $RegKeyPath -ErrorAction Stop
					if ($obj.UninstallString) {
						$guid = ($obj.UninstallString -replace "msiexec\.exe\s/[IX]{1}","")
						if($guid -eq $FolderProductcode){
							Write-Output "Uninstall guid: " + $guid 
							Write-Output "Uninstall version: " + $obj.DisplayVersion 
							Write-Output "Uninstall name: " + $obj.DisplayName 
							Write-Output "Uninstall vendor: " + $obj.Publisher 
							
							$obj = ''
							$uninstall = 1
						}
					}
				} catch {
				}
			}
		}
	}
	else{		
		$FolderProductcode = "$FolderProductcode\$MSIName"
		Write-Output "Check MSI path: " + $FolderProductcode 
		if ($FolderProductcode -like "*\*.msi") {  
			if (Test-Path $FolderProductcode) {
				$uninstall = 1
			}
		}
	}

	# Uninstall
	if($uninstall -eq 1){	    
	
		$logFile = $LogPath+'\'+$AppName+'-'+$AppVersion+'-Uninstall-' + $DataStamp+'.log'
		$arguments = '/q /x ' +$FolderProductcode+ ' /passive ' + $MSIArguments + ' /l*v ' + $logFile

		$output = 'Start-Process "msiexec.exe" -ArgumentList "' + $arguments + '" -PassThru'
		Write-Output "Run: " + $output   

		$Request = Start-Process "msiexec.exe" -ArgumentList "$arguments" -PassThru
						
		$Request | Wait-Process -Timeout $Timeout
		If ($($Request.ExitCode) -eq '0')
		{
			Write-Output "$AppName uninstalled successfully!"
		}
		else{
			Write-Error "$AppName failed to uninstall with exit error: $($Request.ExitCode)"
		}
		Write-Output "Request: $Request"
	}
	else{
		Write-Warning "MSI or productcode not found: $FolderProductcode"
	}
}

if ($Action -eq "install")
{
	Write-Output "Install App - ($AppName)......."  
	
	$FolderProductcode = "$FolderProductcode\$MSIName"
	# Check MSI exists
	Write-Output "Check MSI path: " + $FolderProductcode 
	if (Test-Path $FolderProductcode) {
		$logFile = $LogPath + '\'+$AppName+'-'+$AppVersion+'-Install-' + $DataStamp+'.log'
		$arguments = '/q /i ' + $FolderProductcode + ' ALLUSERS=ALLUSERS /passive ' + $MSIArguments + ' /l*v ' + $logFile

		$output = 'Start-Process "msiexec.exe" -ArgumentList "' + $arguments + '" -PassThru'
		Write-Output "Run: " + $output

		$Request = Start-Process "msiexec.exe" -ArgumentList "$arguments" -PassThru
		
		$Request | Wait-Process -Timeout $Timeout
		If($($Request.ExitCode) -eq '0')
		{
			Write-Output "$AppName installed successfully!"
		}
		else{
			Write-Error "$AppName failed to install with exit error: $($Request.ExitCode)"
		}
	}
	else{
		Write-Error "MSI not found: $FolderProductcode"
	}
}
	
if ($Action -eq "sql")
{
	Write-Output "Run SQL Script - ($AppName)......."  
	
	Write-Output "Provide modul sqlps in powershell"
	import-module "sqlps" -DisableNameChecking

	# Check MSI exists
	if (Test-Path $FolderProductcode) {
		# Check SQL File and run           
		Write-Output "SQL file search: $FolderProductcode\*.slq"
		$files = Get-ChildItem "$FolderProductcode\*.sql" -Recurse | Select-Object Name, Fullname | Sort-Object Name
		foreach  ($file in $files){
			try{
				Write-Output "Script run: invoke-sqlcmd -ServerInstance 'localhost' -Database $DBName -InputFile $file.FullName -QueryTimeout $Timeout -ConnectionTimeout $Timeout  -ErrorAction 'Stop'"
				invoke-sqlcmd -ServerInstance 'localhost' -Database $DBName -InputFile $file.FullName -QueryTimeout $Timeout -ConnectionTimeout $Timeout  -ErrorAction 'Stop'
			}
			catch{
				Write-Error "Error in script: $file.FullName"
				Write-Error $error -foregroundcolor red
			}
		}  
	}
	else{
		Write-Error "SQL path not found: $FolderProductcode"
	} 
}	